#include "ss.h"

double evaluate(SS *prob,double *x)
{
	double value;
	int nvar;

	nvar=prob->n_var;

	value = 100*pow(x[2]-pow(x[2],2),2)+pow(1-x[1],2)+90*pow(x[4]-pow(x[3],2),2)+
				    pow(1-x[3],2)+10.1*pow(x[2]-1,2)+10.1*pow(x[4]-1,2)+19.8*(x[2]-1)*(x[4]-1);

	return value;
}


void main(void)	
{
	int MaxIter,Iter,i,nvar,b1,b2,PSize,LocalSearch;
	double *best,best_value,LowerBound,UpperBound;
	SS *prob;

	/* Parameters Defined by the User */ 
	MaxIter    =   2; /* Number of global iterations         */
	nvar       =   4; /* Number of variables (size of x)     */
	LowerBound = -10; /* Lower Bound of all variables		 */	
	UpperBound =  10; /* Upper Bound of all variables        */
	b1		   =  10; /* Size of reference set 1 (quality)   */
	b2		   =  10; /* Size of reference set 2 (diversity) */
 	PSize	   = 100; /* Size of the diversification generator population   */
	LocalSearch=   1; /* =1 Nelder and Mead Simplex Method activated,=0 OFF */

	prob=DataStructures_init(nvar,b1,b2,PSize,LocalSearch);

	/* Assign Variable Bounds */
	for(i=1;i<=nvar;i++)
	{
		prob->low[i]  = LowerBound; 
		prob->high[i] = UpperBound;
	}

	/* Build Reference Set */
	Initiate_RefSet(prob);

	/* Perform Search */
	for(Iter=1;Iter<=MaxIter;Iter++)
	{
		if(prob->new_elements)
			Combine_RefSet(prob);
		else
		{	
			Update_RefSet2(prob);
			Combine_RefSet(prob);
		}
	}

	/*** Best Solution ***/
	best_value = prob->value1[prob->order1[1]];

	best = SSallocate_double_array(nvar);
	for(i=1;i<=nvar;i++) best[i]=prob->RefSet1[prob->order1[1]][i];

	printf("\n\nBest Solution Found. Value = %f\n\n",best_value);
	for(i=1;i<=nvar;i++) printf("solution[%d] = %f\n",i,best[i]);
	free(best);

	Free_DataStructures(prob);
}


